#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <termios.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/ioctl.h>

extern int      ps_fd;

int
siocons_send_command(const char *str)
{
	u_char buf[100], dat = 0x0d;
	int n;
	fd_set          rset;
	struct timeval  timeout = {0,100000};

	if (write(ps_fd, str, strlen(str)) < 0 
		|| write(ps_fd, &dat, 1) < 0) {
		fprintf(stderr,"siocons_send_command: [%s] %s\n", 
				str, strerror(errno));
	}

	FD_ZERO(&rset);
	do {
		FD_SET(ps_fd, &rset);
		if ((n = select(ps_fd + 1, &rset, NULL, NULL, &timeout)) < 0) {
			perror("select");
			exit(1);
		}
		if (FD_ISSET(ps_fd, &rset))
			read(ps_fd, buf, sizeof(buf));
	} while(n != 0);
end:
	return;
}

int
siocons_send_command_verbose(const char *str)
{
	u_char buf[100], dat = 0x0d;
	int n, n2;
	fd_set          rset;
	struct timeval  timeout = {0,100000};

	if (write(ps_fd, str, strlen(str)) < 0 
		|| write(ps_fd, &dat, 1) < 0) {
		fprintf(stderr,"siocons_send_command: [%s] %s\n", 
				str, strerror(errno));
	}

	FD_ZERO(&rset);
	do {
		FD_SET(ps_fd, &rset);
		if ((n = select(ps_fd + 1, &rset, NULL, NULL, &timeout)) < 0) {
			perror("select");
			exit(1);
		}
		if (FD_ISSET(ps_fd, &rset)) {
			n2 = read(ps_fd, buf, sizeof(buf));
			write(STDOUT_FILENO, buf, n2);
		}
	} while(n != 0);
end:
	return;
}

int 
read_till_lf(char *buf)
{
	int             n, len = 0;
	char            c;
	while (1) {
		if ((n = read(ps_fd, &c, 1)) < 0) {
			perror("read");
			return -1;
		}
		if (c == 0x0d || c == 0x0a) {
			buf[len] = '\0';
			return len;
		} else
			buf[len++] = c;
	}
}

static int
handshake_send(u_char dat)
{
	int             n, set, set2;
	int             wait_time = 0;
	struct timeval  t = {0, 100};

	/* wait cts */
	while (1) {
		ioctl(ps_fd, TIOCMGET, &set);
		if (set & TIOCM_CTS)
			break;
		
		if (wait_time++ >= 100000) {
			fprintf(stderr, "handshake send timeout!\n");
			return -1;
		}

		if ((n = select(0, NULL, NULL, NULL, &t)) < 0) {
			perror("select");
			return -1;
		}
	}

	/* write */
	if ((n = write(ps_fd, &dat, 1)) < 0)
		perror("write");

	return n;
}

static int 
handshake_send2(u_char dat)
{
	int             n, set;

	set = TIOCM_RTS;
	ioctl(ps_fd, TIOCMBIS, &set);

	/* busy loop */
	while (1) {
		ioctl(ps_fd, TIOCMGET, &set);
		if (set & TIOCM_CTS)
			break;
	}

	/* write */
	if ((n = write(ps_fd, &dat, 1)) < 0)
		perror("write");

//	set = TIOCM_RTS;
//	ioctl(ps_fd, TIOCMBIC, &set);
//	ioctl(ps_fd, TIOCMBIS, &set);

	return n;
}

static int 
handshake_recv(u_char * dat)
{
	int n, set;

	/* handshake */
	set = TIOCM_RTS;
	ioctl(ps_fd, TIOCMBIC, &set);
	ioctl(ps_fd, TIOCMBIS, &set);

	if ((n = read(ps_fd, dat, 1)) < 0)
		perror("read");

	return n;
}

static int
handshake_recv2(u_char *dat)
{
    int n, set;

    /* handshake */
    set = TIOCM_RTS;
    ioctl(ps_fd,TIOCMBIS,&set);

    /* read */
    if ((n = read(ps_fd,dat,1)) < 0)
        perror("read");

    /* handshake */
//  set = TIOCM_RTS;
//  ioctl(ps_fd,TIOCMBIC,&set);

    return n;
}

void
siocons_load(u_long addr, u_long size, FILE *fp)
{
	u_long data_size2;

	u_char buf[0x800], *p;
	int i, j, n;
	int cnt, set;
	u_char sum, dat;

	data_size2 = ((size + 0x7ff) / 0x800) * 0x800;
	cnt = data_size2 / 0x800;

	if (write(ps_fd, "bwr\x0d", 4) < 0) {
		perror("write");
		exit(1);
	}
	do {
		if ((n = read_till_lf(buf)) < 0) {
			fprintf(stderr, "Binary transfer protocol error.\n");
			goto end;
		}
	} while (n != 6 || strncmp(buf, "binary", 6));


	/* handshake */
	{
		set = TIOCM_RTS;
		ioctl(ps_fd, TIOCMBIS, &set);
	}

	{
		handshake_send(0x01);
		handshake_send((addr >> 24) & 0xff);
		handshake_send((addr >> 16) & 0xff);
		handshake_send((addr >>  8) & 0xff);
		handshake_send((addr >>  0) & 0xff);

		handshake_send((size >> 24) & 0xff);
		handshake_send((size >> 16) & 0xff);
		handshake_send((size >>  8) & 0xff);
		handshake_send((size >>  0) & 0xff);
	}		

	for (i = 0; i < cnt; i++) {
		
		handshake_send2(0x02);
		n = fread(buf,0x800,1,fp);

	retry:
		p = buf;
		sum = 0;

		for (j = 0; j < 0x800; j++, p++) {

			if (handshake_send(*p) <= 0)
				goto end;
			sum += *p;
		}

		fprintf(stdout, "\r%08x byte transfered : sum[%02x]",
				(i+1)*0x800, sum);
		fflush(stdout);

		handshake_send2(sum);	/* sum */
		handshake_recv2(&dat);
		if (dat != 0x59)
			goto end;
	}

	handshake_send2(0x0d);
	if ((n = read_till_lf(buf)) != 10 || strncmp(buf, "end binary", 10)) {
		
		fprintf(stderr, "Binary transfer protocol error.\n");
		goto end;
	}

 end:
	return;
}

void
siocons_save(u_long addr, u_long size, FILE *fp)
{
	u_char buf[0x800], *p;
	u_long size2;
	int i, j, n;
	int cnt, set;
	u_char sum, dat;

	size2 = ((size + 0x7ff) / 0x800) * 0x800;
	cnt = size2 / 0x800;

	if (write(ps_fd, "brd\015", 4) < 0) {
		perror("write");
		exit(1);
	}

	do {
		if ((n = read_till_lf(buf)) < 0) {
			fprintf(stderr, "Binary transfer protocol error.\n");
			goto end;
		}
/*		fprintf(stderr, "%d [%s]\n", n, buf); */
	} while (n != 6 || strncmp(buf, "binary", 6));

	{
		handshake_send(0x01);
		handshake_send((addr >> 24) & 0xff);
		handshake_send((addr >> 16) & 0xff);
		handshake_send((addr >>  8) & 0xff);
		handshake_send((addr >>  0) & 0xff);

		handshake_send((size2 >> 24) & 0xff);
		handshake_send((size2 >> 16) & 0xff);
		handshake_send((size2 >>  8) & 0xff);
		handshake_send((size2 >>  0) & 0xff);
	}		

	cnt = size2 / 0x800;

	for (i = 0; i < cnt;i++) {
		
		handshake_send(0x02);
		sum = 0;
		p = buf;

		for (j = 0; j < 0x800; j++, p++) {

			if (handshake_recv(p) <= 0)
				goto end;
			sum += *p;

			if (j % 0x40 == 0) {
				fprintf(stdout, "\r%02d:%03x byte transferd....", i,j + 0x40);
				fflush(stdout);
			}
		}

		if (handshake_recv(&dat) <= 0)
			goto end;
		fprintf(stdout, "chksum: %02x\n", dat);

		if (dat != sum)
			fprintf(stderr, "warn: chksum err : %02x\n", sum);

		handshake_send(0x59);	/* OK */

		if (i < (cnt-1)) {
			fwrite(buf, 0x800, 0x1, fp);
			size -= 0x800;
		} else {
			fwrite(buf, size, 0x1, fp);
		}
	}

	handshake_send(0x99);	/* all end */

	if ((n = read_till_lf(buf)) != 10 || strncmp(buf, "end binary", 10)) {
		fprintf(stderr, "Binary transfer protocol error.\n");
		goto end;
	}	

 end:
	return;

}

int  script_end_flag;

void
siocons_autofile(FILE *fp)
{
	char buf[1024+1];

	script_end_flag = 0;
	while (!script_end_flag && fgets(buf, sizeof(buf)-1, fp) != NULL ) {
		char *p = buf;
		while (isspace(*p)) p++;
		if (!strncmp(p,"local ",6) || !strncmp(p,"LOCAL ",6)) {
			if (siocons_parse(p + 6))
				goto end;
		} else {
			siocons_send_command_verbose(p);
		}
	}
 end:
	return;
}

FILE *auto_load_fp;

void
siocons_local(void)
{
	char buf[1024+1];

	script_end_flag = 0;
	auto_load_fp    = NULL;
	fprintf(stdout, "\n");
	do {
		fprintf(stdout, "local>");
		if (fgets(buf, sizeof(buf)-1, stdin) == NULL)
			break;
		siocons_parse(buf);

		if (auto_load_fp != NULL) {
			siocons_autofile(auto_load_fp);
			fclose(auto_load_fp);
			auto_load_fp = NULL;
			script_end_flag = 0;
		}
	} while (!script_end_flag);
}
