#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/stat.h>

#include "siocons.h"

void 
help(void)
{
    char help[] = "siocons local mode help messages\n"
		"-------------------------------------------------\n"
        "help                   show this messages.\n"
        "dos <arg>              execute shell.\n"
        "dload file addr        load file to memory.\n"
        "load file              load executable file.\n"
        "dsave file addr size   save memory to file.\n"
        "beep                   beep buzzer.\n"
        "pause                  pause and wait key press.\n"
        "echo str               echo strings.\n"
        "sleep sec              wait sec [sec].\n"
        "wait-prompt            wait monitor prompt.\n"
        "quit                   quit siocons.\n"
        "end                    end local mode.\n"
		"-------------------------------------------------\n";
	fprintf(stdout,help);
}

void 
shell(char *str)
{
	if (system(str) < 0)
		perror("shell");
}

extern FILE *auto_load_fp;

void 
autofile(char *str)
{
	FILE *fp;
	if ((fp = fopen(str,"r")) == NULL) {
		perror(str);
	} else
		auto_load_fp = fp;
}

void 
log_off(void)
{
}

void 
log_on(char *str)
{
}

void 
beep(void)
{
	fprintf(stdout,"\007");
}

void 
sio_pause(void)
{
	u_char buf[100];
	int             n;
	fd_set          rset;

	FD_ZERO(&rset);
	FD_SET(STDIN_FILENO, &rset);
	if ((n = select(STDIN_FILENO+1, &rset, NULL, NULL, NULL)) < 0) {
		perror("select");
		exit(1);
	}
	if (FD_ISSET(STDIN_FILENO, &rset)) {
		read(STDIN_FILENO, buf, sizeof(buf));
	}
}

void 
echo(char *str)
{
	fprintf(stdout, "[%s]\n", str);
}

void 
sio_sleep(int time)
{
	sleep(time);
}

void 
sio_wait(void)
{
	u_char buf[0x800], dat = 0x0d;
	int n;
	fprintf(stdout,"waiting...\n");
	do {
		if ((n = read_till_lf(buf)) < 0) {
			fprintf(stderr, "Binary transfer protocol error.\n");
			return;
		}
	} while (n != 2 || strncmp(buf, "OK", 2));
}

void 
again(void)
{
}

void 
load(char *name)
{
	ecoff_load(name, 1);
}

void 
dload(char *name, u_long addr)
{
	FILE *fp;
	struct stat sb;

	if (stat(name, &sb) < 0) {
		perror("stat:");
		return;
	}
	if((fp = fopen(name,"r")) == NULL) {
		perror(name);
		return;
	}

	fprintf(stdout, "dload: %s addr: %08lx size: %08lx\n",
			name, addr, sb.st_size);

	siocons_load(addr, sb.st_size, fp);
	fprintf(stdout, "\ndload DONE.\n");

	fclose(fp);
}

void 
dsave(char *name, u_long addr, u_long size)
{
	FILE *fp;

	if((fp = fopen(name,"w")) == NULL) {
		perror(name);
		return;
	}
	fprintf(stdout, "dsave: %s addr: %08lx size: %08lx\n",
			name, addr, size);

	siocons_save(addr, size, fp);
	fprintf(stdout, "dsave DONE.\n");
	fclose(fp);
}
