#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>

extern int script_end_flag;

#define DEBUG 0

static const char *token;

void
token_start(const char *str)
{
	token = str;
}

char    token_str[1024];
u_long  token_num;

int
token_none(void)
{
	while (isspace(*token)) token++;
	if (*token != '\0') {
		return -1;
	}
	else
		return 0;
}

int
token_command(void)
{
	char *q = token_str;

	while (isspace(*token)) token++;
	while (*token != '\0' && !isspace(*token)) {
		if (isalnum(*token) || *token == '_' || *token == '-') {
			*q++ = tolower(*token++);
		} else
			return -1;
	}
	*q = '\0';
#if DEBUG
	fprintf(stderr,"cmd[%s]\n",token_str);
#endif
	return 0;
}

int
token_string(void)
{
	char *q = token_str;

	while (isspace(*token)) token++;
	while (*token != '\0' && !isspace(*token)) {
		if (*token == '\"') {
			token++;
			while (*token != '\"' && *token !='\0') {
				if (*token == '\\') {
					if (*(token+1) != '\0' ) {
						*q++ = *(token+1);
						token += 2;
					} else
						*q++ = *token++;
				}else
					*q++ = *token++;
			}
			goto end;
		} else
			*q++ = *token++;
	}
 end:
	*q = '\0';
#if DEBUG
	fprintf(stderr,"str[%s]\n",token_str);
#endif
	return 0;
}

int
token_hex(void)
{
	token_num = 0;

	while (isspace(*token)) token++;
	if (*token == '0' && toupper(*(token+1)) == 'X')
		token += 2;
	while (*token != '\0' && !isspace(*token)) {
		if (isxdigit(*token)) {
			int num;
			if (isdigit(*token))
				num = *token - '0';
			else
				num = toupper(*token) - 'A' + 10;
			token_num = token_num * 16 + num;
			token++;
		} else
			return -1;
	}
#if DEBUG
	fprintf(stderr,"hex[%x]\n",token_num);
#endif
	return 0;
}	

int
token_number(void)
{
	token_num = 0;
	while (isspace(*token)) token++;
	while (*token != '\0' && !isspace(*token)) {
		if (isdigit(*token)) {
			token_num = token_num * 10 + (*token-'0');
			token++;
		} else
			return -1;
	}
#if DEBUG	
	fprintf(stderr,"num[%d]\n",token_num);
#endif
	return 0;
}

#define MATCH(s) (!strncmp(token_str,s,sizeof(s)-1))

int
siocons_parse(char *str)
{
	token_start(str);
	if (token_command())
		goto err;

	if (token_str[0] == '\0')
		return ;

	if (MATCH("dload")) {
		if (token_string())
			goto err;
		if (token_hex())
			goto err;
		if (token_none())
			goto err;
		dload(token_str, token_num);
	}

	else if (MATCH("dsave")) {
		int addr;
		if (token_string())
			goto err;
		if (token_hex())
			goto err;
		addr = token_num;
		if (token_hex())
			goto err;
		if (token_none())
			goto err;
		dsave(token_str, addr, token_num);
	}

	else if (MATCH("load")) {
		if (token_string())
			goto err;
		if (token_none())
			goto err;
		load(token_str);
	}		

	else if (MATCH("shell") || MATCH("dos")) {
		if (token_string())
			goto err;
		if (token_none())
			goto err;
		shell(token_str);
	}

	else if (MATCH("echo")) {
		if (token_string())
			goto err;
		if (token_none())
			goto err;
		echo(token_str);
	}

	else if (MATCH("sleep")) {
		if (token_number())
			goto err;
		if (token_none())
			goto err;
		sio_sleep(token_num);
	}

	else if (MATCH("auto")) {
		if (token_string())
			goto err;
		else {
			autofile(token_str);
		}
	} 

	else if (MATCH("beep")) {
		if (token_none())
			goto err;
		beep();
	}

	else if (MATCH("pause")) {
		if (token_none())
			goto err;
		sio_pause();
	}

	else if (MATCH("wait") || MATCH("wait-prompt")) {
		if (token_none())
			goto err;
		sio_wait();
	}

	else if (MATCH("auto") || MATCH("auto-again")) {
		if (!token_none())
			goto err;
		again();
	}

	else if (MATCH("help")) {
		if (token_none())
			goto err;
		help();
	}

	else if (MATCH("end")) {
		if (token_none())
			goto err;
		script_end_flag = 1;
	}

	else if (MATCH("quit")) {
		if (token_none())
			goto err;
		exit(0);
	}
	else
		fprintf(stderr,"no such command\n");
	return 0;

 err:
	fprintf(stderr,"syntax error: %s \n", str);
	return -1;
}
